package com.mycompany.hunter_a;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.sposh.engine.VariableContext;
import cz.cuni.amis.pogamut.sposh.executor.ActionResult;
import cz.cuni.amis.pogamut.sposh.executor.PrimitiveInfo;
import cz.cuni.amis.pogamut.sposh.executor.StateAction;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.utils.collections.MyCollections;
import cz.cuni.amis.utils.flag.FlagListener;
import java.util.logging.Level;

/**
 * Action that causes bot to run around the map looking for medkits.
 * @author Honza
 */
@PrimitiveInfo(name = "runAroundMap", description = "Run around map.")
public class RunAroundMap extends StateAction<HunterContext> {

    public RunAroundMap(HunterContext ctx) {
        super("runAroundMap", ctx);
    }
    
    //called when this action is scheduled to run after some other different action
    @Override
    public void init(VariableContext params) {
        //setting non-flooding log level
        ctx.getPathExecutor().getLog().setLevel(Level.WARNING);
        
        //System.out.println("RunAroundMap.INIT");
        
        //adding listener for path executor states - CAREFULL either check if we are listening or remove the listener in DONE phase - otherwise 
        //we would add more and more listeners each time this action would be restarted 
        ctx.getPathExecutor().getState().addStrongListener(new FlagListener<IPathExecutorState>() {
            @Override
            public void flagChanged(IPathExecutorState t) {
                if (t.getState() == PathExecutorState.STUCK || t.getState() == PathExecutorState.TARGET_REACHED || t.getState() == PathExecutorState.PATH_COMPUTATION_FAILED ) {
                    //reset and pick new nav point
                    ctx.getPathExecutor().stop();
                    ctx.targetNav = MyCollections.getRandom(ctx.getWorld().getAll(NavPoint.class).values());               
                }                
            }
        });
    }

    //called each time this action is scheduled to run if previous action equals to this action
    @Override
    public ActionResult run(VariableContext params) {
        //System.out.println("RunAroundMap.RUN");
        if (ctx.getPathExecutor().isExecuting()) {
                       
        } else {
            ctx.targetNav = MyCollections.getRandom(ctx.getWorld().getAll(NavPoint.class).values());
            ctx.getPathExecutor().followPath(ctx.getFwMap().computePath(ctx.getInfo().getNearestNavPoint(),ctx.targetNav));                    
        }
        /*
        if (ctx.targetNav != null) {
           if (ctx.targetNav.getLocation().getDistance(ctx.getInfo().getLocation()) <= 100) {
                ctx.targetNav = MyCollections.getRandom(ctx.getWorld().getAll(NavPoint.class).values());                        
           } 
           
           ctx.getNavigation().navigate(ctx.targetNav);
        } else {
           ctx.targetNav = MyCollections.getRandom(ctx.getWorld().getAll(NavPoint.class).values());                        
        }*/
        return ActionResult.RUNNING;
    }

    // called when some other action was scheduled to run after this action
    @Override
    public void done(VariableContext params) {
        //System.out.println("RunAroundMap.DONE");
    	// stopping path executor
        ctx.getPathExecutor().stop();
        // removing listener
        ctx.getPathExecutor().getState().removeAllListeners();
    }
}
